/**************************************************
opengate server
 module for Getting Parameters 

Copyright (C) 2005 Yoshiaki Watanabe

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: watanaby@is.saga-u.ac.jp
**************************************************/

#include "opengatesrv.h"

static char *dbBuffer=NULL; /* malloc-ed buffer for config file */

/****************************************/
/* conf is copied to malloc-ed DbBuffer */
/****************************************/
int createDbBuffer(char* serverid)
{
  static int accessed=FALSE; /* is the file already accessed */
  char *dbArray[2];  /* config file names */

  /* to avoid multiple malloc, return at second call */
  if(accessed) return 0;
  accessed = TRUE;

  /* if no string, set default */
  if(serverid==NULL){
    serverid="default";
  }

  /* pickup server info into dbBuffer from config file */
  dbArray[0]=CONFIGFILE;
  dbArray[1]=NULL;

  if(cgetent(&dbBuffer, dbArray, serverid)<0){
    if(cgetent(&dbBuffer, dbArray, "default")<0){
      err_msg("ERR in get-param: cannot get entry from conf file");
    }
  }

  return 0;
}
/**********************************/
/* release malloc-ed DbBuffer     */
/**********************************/
int releaseDbBuffer(void)
{
  free(dbBuffer);
  return 0;
}

/***************************/
/* get auth server address */
/***************************/
char* getAuthAddress(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static char* mstr=NULL;    /* maloc-ed area for param string */

  /* to avoid multiple malloc, return the previous string */
  if(accessed) return mstr;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    mstr=NULL;
    return mstr;
  }

  /* get address */
  if(cgetstr(dbBuffer, "address", &mstr)<0){
    mstr=NULL;
  }
  return mstr;
}


/***************************/
/* get auth server port    */
/***************************/
char* getAuthPort(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static char* mstr=NULL;    /* maloc-ed area for param string */

  /* to avoid multiple malloc, return the previous string */
  if(accessed) return mstr;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    mstr=NULL;
    return mstr;
  }

  /* get port */
  if(cgetstr(dbBuffer, "port", &mstr)<0){
    mstr=NULL;
  }
  return mstr;
}

/***************************/
/* get auth protocol       */
/***************************/
char* getAuthProtocol(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static char* mstr=NULL;    /* maloc-ed area for param string */

  /* to avoid multiple malloc, return the previous string */
  if(accessed) return mstr;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    mstr=DEFAULTPROTO;
    return mstr;
  }

  /* get protocol */
  if(cgetstr(dbBuffer, "protocol",&mstr)<0){
    mstr=DEFAULTPROTO;
  }
  return mstr;
}

/***************************/
/* get PAM service name    */
/***************************/
char* getPamServiceName(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static char* mstr=NULL;    /* maloc-ed area for param string */

  /* to avoid multiple malloc, return the previous string */
  if(accessed) return mstr;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    mstr=PAMSERVICENAME;
    return mstr;
  }

  /* get servicename */
  if(cgetstr(dbBuffer, "servicename",&mstr)<0){
    mstr=PAMSERVICENAME;
  }
  return mstr;
}

/***************************/
/* get RADIUS config file  */
/***************************/
char* getRadiusConfFile(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static char* mstr=NULL;    /* maloc-ed area for param string */

  /* to avoid multiple malloc, return the previous string */
  if(accessed) return mstr;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    mstr=RADIUSCONF;
    return mstr;
  }

  /* get conffile */
  if(cgetstr(dbBuffer, "conffile",&mstr)<0){
    mstr=RADIUSCONF;
  }
  return mstr;
}


/***************************/
/* get DurationDefault     */
/***************************/
int getDurationDefault(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static long num;    /* save numeric parameter */
  char* mstr;         /* string from capdb */
  char* endp;         /* pointer indicating conversion termination */

  /* to avoid multiple access, return the previous one */
  if(accessed) return num;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    num=DURATIONDEFAULT;
    return num;
  }

  /* get string */
  if(cgetstr(dbBuffer, "durationdefault",&mstr)<0){
    num=DURATIONDEFAULT;
  }else{
    num=strtol(mstr,&endp,10);
    free(mstr);
  }

  return num;
}

/***************************/
/* get DurationMax         */
/***************************/
int getDurationMax(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static long num;    /* save numeric parameter */
  char* mstr;         /* string from capdb */
  char* endp;         /* pointer indicating conversion termination */

  /* to avoid multiple access, return the previous one */
  if(accessed) return num;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    num=DURATIONMAX;
    return num;
  }

  /* get string */
  if(cgetstr(dbBuffer, "durationmax",&mstr)<0){
    num=DURATIONMAX;
  }else{
    num=strtol(mstr,&endp,10);
    free(mstr);
  }

  return num;
}

/***************************/
/* get ActiveCheckInterval */
/***************************/
int getActiveCheckInterval(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static long num;    /* save numeric parameter */
  char* mstr;         /* string from capdb */
  char* endp;         /* pointer indicating conversion termination */

  /* to avoid multiple access, return the previous one */
  if(accessed) return num;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    num=ACTIVECHECKINTERVAL;
    return num;
  }

  /* get string */
  if(cgetstr(dbBuffer, "activecheckinterval",&mstr)<0){
    num=ACTIVECHECKINTERVAL;
  }else{
    num=strtol(mstr,&endp,10);
    free(mstr);
  }
  return num;
}

/***************************/
/* get NoReplyMax */
/***************************/
int getNoReplyMax(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static long num;    /* save numeric parameter */
  char* mstr;         /* string from capdb */
  char* endp;         /* pointer indicating conversion termination */

  /* to avoid multiple access, return the previous one */
  if(accessed) return num;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    num=NOREPLYMAX;
    return num;
  }

  /* get string */
  if(cgetstr(dbBuffer, "noreplymax",&mstr)<0){
    num=NOREPLYMAX;
  }else{
    num=strtol(mstr,&endp,10);
    free(mstr);
  }

  return num;
}

/***************************/
/* get NoPacketInterval    */
/***************************/
int getNoPacketInterval(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static long num;    /* save numeric parameter */
  char* mstr;         /* string from capdb */
  char* endp;         /* pointer indicating conversion termination */

  /* to avoid multiple access, return the previous one */
  if(accessed) return num;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    num=NOPACKETINTERVAL;
    return num;
  }

  /* get string */
  if(cgetstr(dbBuffer, "nopacketintereval",&mstr)<0){
    num=NOPACKETINTERVAL;
  }else{
    num=strtol(mstr,&endp,10);
    free(mstr);
  }

  return num;
}


/***************************/
/* get IpfwMin             */
/***************************/
int getIpfwMin(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static long num;    /* save numeric parameter */
  char* mstr;         /* string from capdb */
  char* endp;         /* pointer indicating conversion termination */

  /* to avoid multiple access, return the previous one */
  if(accessed) return num;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    num=IPFWMIN;
    return num;
  }

  /* get string */
  if(cgetstr(dbBuffer, "ipfwmin",&mstr)<0){
    num=IPFWMIN;
  }else{
    num=strtol(mstr,&endp,10);
    free(mstr);
  }

  return num;
}

/***************************/
/* get IpfwMax             */
/***************************/
int getIpfwMax(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static long num;    /* save numeric parameter */
  char* mstr;         /* string from capdb */
  char* endp;         /* pointer indicating conversion termination */

  /* to avoid multiple access, return the previous one */
  if(accessed) return num;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    num=IPFWMAX;
    return num;
  }

  /* get string */
  if(cgetstr(dbBuffer, "ipfwmax",&mstr)<0){
    num=IPFWMAX;
  }else{
    num=strtol(mstr,&endp,10);
    free(mstr);
  }

  return num;
}

/***************************/
/* get IpfwInterval        */
/***************************/
int getIpfwInterval(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static long num;    /* save numeric parameter */
  char* mstr;         /* string from capdb */
  char* endp;         /* pointer indicating conversion termination */

  /* to avoid multiple access, return the previous one */
  if(accessed) return num;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    num=IPFWINTERVAL;
    return num;
  }

  /* get string */
  if(cgetstr(dbBuffer, "ipfwinterval",&mstr)<0){
    num=IPFWINTERVAL;
  }else{
    num=strtol(mstr,&endp,10);
    free(mstr);
  }

  return num;
}


/***************************/
/* get PortMin             */
/***************************/
int getPortMin(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static long num;    /* save numeric parameter */
  char* mstr;         /* string from capdb */
  char* endp;         /* pointer indicating conversion termination */

  /* to avoid multiple access, return the previous one */
  if(accessed) return num;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    num=PORTMIN;
    return num;
  }

  /* get string */
  if(cgetstr(dbBuffer, "portmin",&mstr)<0){
    num=PORTMIN;
  }else{
    num=strtol(mstr,&endp,10);
    free(mstr);
  }

  return num;
}

/***************************/
/* get PortMax             */
/***************************/
int getPortMax(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static long num;    /* save numeric parameter */
  char* mstr;         /* string from capdb */
  char* endp;         /* pointer indicating conversion termination */

  /* to avoid multiple access, return the previous one */
  if(accessed) return num;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    num=PORTMAX;
    return num;
  }

  /* get string */
  if(cgetstr(dbBuffer, "portmax",&mstr)<0){
    num=PORTMAX;
  }else{
    num=strtol(mstr,&endp,10);
    free(mstr);
  }

  return num;
}


/***************************/
/* get EnableMacAddr       */
/***************************/
int getEnableMacAddr(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static long num;    /* save numeric parameter */
  char* mstr;         /* string from capdb */
  char* endp;         /* pointer indicating conversion termination */

  /* to avoid multiple access, return the previous one */
  if(accessed) return num;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    num=ENABLEMACADDR;
    return num;
  }

  /* get string */
  if(cgetstr(dbBuffer, "enablemacaddr",&mstr)<0){
    num=ENABLEMACADDR;
  }else{
    num=strtol(mstr,&endp,10);
    free(mstr);
  }

  return num;
}

/***************************/
/* getFwScriptPath         */
/***************************/
char* getFwScriptPath(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static char* mstr=NULL;    /* maloc-ed area for param string */

  /* to avoid multiple malloc, return the previous string */
  if(accessed) return mstr;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    mstr=FWSCRIPTPATH;
    return mstr;
  }

  /* get string */
  if(cgetstr(dbBuffer, "fwscriptpath",&mstr)<0){
    mstr=FWSCRIPTPATH;
  }
  return mstr;
}


/***************************/
/* getFwScript             */
/***************************/
char* getFwScript(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static char* mstr=NULL;    /* maloc-ed area for param string */

  /* to avoid multiple malloc, return the previous string */
  if(accessed) return mstr;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    mstr=FWSCRIPT;
    return mstr;
  }

  /* get string */
  if(cgetstr(dbBuffer, "fwscript",&mstr)<0){
    mstr=FWSCRIPT;
  }
  return mstr;
}

/***************************/
/* getUseFwScript          */
/***************************/
int getUseFwScript(void)
{
  static int accessed=FALSE; /* is the parameter already accessed */
  static long num;    /* save numeric parameter */
  char* mstr;         /* string from capdb */
  char* endp;         /* pointer indicating conversion termination */

  /* to avoid multiple access, return the previous one */
  if(accessed) return num;
  accessed = TRUE;

  /* If no buffer, then error */ 
  if(dbBuffer==NULL){
    err_msg("ERR in get-param: db buffer is not created");
    num=USEFWSCRIPT;
    return num;
  }

  /* get string */
  if(cgetstr(dbBuffer, "usefwscript",&mstr)<0){
    num=USEFWSCRIPT;
  }else{
    num=strtol(mstr,&endp,10);
    free(mstr);
  }

  return num;
}



/**********************************/
/* Debug routines                 */
/**********************************/
int CreateDbBuffer(char* serverid)
{
  int ret;

  if(DEBUG) err_msg("DEBUG:=>ceateDbBuffer(%s)",serverid);
  ret=createDbBuffer(serverid);
  if(DEBUG) err_msg("DEBUG:(%d)<=ceateDbBuffer()",ret);
  return ret;
}

int ReleaseDbBuffer(void)
{
  int ret;

  if(DEBUG) err_msg("DEBUG:=>releaseDbBuffer()");
  ret=releaseDbBuffer();
  if(DEBUG) err_msg("DEBUG:(%d)<=releaseDbBuffer()",ret);
  return ret;
}

char* GetAuthAddress(void)
{
  char* ret;
  if(DEBUG) err_msg("DEBUG:=>getAuthAddress()");
  ret=getAuthAddress();
  if(DEBUG) err_msg("DEBUG:(%s)<=getAuthAddress()",ret);
  return ret;
}

char* GetAuthPort(void)
{
  char* ret;
  if(DEBUG) err_msg("DEBUG:=>getAuthPort()");
  ret=getAuthPort();
  if(DEBUG) err_msg("DEBUG:(%s)<=getAuthPort()",ret);
  return ret;
}

char* GetAuthProtocol(void)
{
  char* ret;
  if(DEBUG) err_msg("DEBUG:=>getAuthProtocol()");
  ret=getAuthProtocol();
  if(DEBUG) err_msg("DEBUG:(%s)<=getAuthProtocol()",ret);
  return ret;
}

char* GetPamServiceName(void)
{
  char* ret;
  if(DEBUG) err_msg("DEBUG:=>getPamServiceName()");
  ret=getPamServiceName();
  if(DEBUG) err_msg("DEBUG:(%s)<=getPamServiceName()",ret);
  return ret;
}

char* GetRadiusConfFile(void)
{
  char* ret;
  if(DEBUG) err_msg("DEBUG:=>getRadiusConfFile()");
  ret=getRadiusConfFile();
  if(DEBUG) err_msg("DEBUG:(%s)<=getRadiusConfFile()",ret);
  return ret;
}

int GetDurationDefault(void)
{
  int ret;
  if(DEBUG) err_msg("DEBUG:=>getDurationDefault()");
  ret=getDurationDefault();
  if(DEBUG) err_msg("DEBUG:(%d)<=getDurationDefault()",ret);
  return ret;
}

int GetDurationMax(void)
{
  int ret;
  if(DEBUG) err_msg("DEBUG:=>getDurationMax()");
  ret=getDurationMax();
  if(DEBUG) err_msg("DEBUG:(%d)<=getDurationMax()",ret);
  return ret;
}

int GetActiveCheckInterval(void)
{
  int ret;
  if(DEBUG) err_msg("DEBUG:=>getActiveCheckInterval()");
  ret=getActiveCheckInterval();
  if(DEBUG) err_msg("DEBUG:(%d)<=getActiveCheckInterval()",ret);
  return ret;
}

int GetNoReplyMax(void)
{
  int ret;
  if(DEBUG) err_msg("DEBUG:=>getNoReplyMax()");
  ret=getNoReplyMax();
  if(DEBUG) err_msg("DEBUG:(%d)<=getNoReplyMax()",ret);
  return ret;
}

int GetNoPacketInterval(void)
{
  int ret;
  if(DEBUG) err_msg("DEBUG:=>getNoPacketInterval()");
  ret=getNoPacketInterval();
  if(DEBUG) err_msg("DEBUG:(%d)<=getNoPacketInterval()",ret);
  return ret;
}

int GetIpfwMin(void)
{
  int ret;
  if(DEBUG) err_msg("DEBUG:=>getIpfwMin()");
  ret=getIpfwMin();
  if(DEBUG) err_msg("DEBUG:(%d)<=getIpfwMin()",ret);
  return ret;
}

int GetIpfwMax(void)
{
  int ret;
  if(DEBUG) err_msg("DEBUG:=>getIpfwMax()");
  ret=getIpfwMax();
  if(DEBUG) err_msg("DEBUG:(%d)<=getIpfwMax()",ret);
  return ret;
}

int GetIpfwInterval(void)
{
  int ret;
  if(DEBUG) err_msg("DEBUG:=>getIpfwInterval()");
  ret=getIpfwInterval();
  if(DEBUG) err_msg("DEBUG:(%d)<=getIpfwInterval()",ret);
  return ret;
}

int GetPortMin(void)
{
  int ret;
  if(DEBUG) err_msg("DEBUG:=>getPortMin()");
  ret=getPortMin();
  if(DEBUG) err_msg("DEBUG:(%d)<=getPortMin()",ret);
  return ret;
}

int GetPortMax(void)
{
  int ret;
  if(DEBUG) err_msg("DEBUG:=>getPortMax()");
  ret=getPortMax();
  if(DEBUG) err_msg("DEBUG:(%d)<=getPortMax()",ret);
  return ret;
}

int GetEnableMacAddr(void)
{
  int ret;
  if(DEBUG) err_msg("DEBUG:=>getEnableMacAddr()");
  ret=getEnableMacAddr();
  if(DEBUG) err_msg("DEBUG:(%d)<=getEnableMacAddr()",ret);
  return ret;
}

char* GetFwScriptPath(void)
{
  char* ret;
  if(DEBUG) err_msg("DEBUG:=>getFwScriptPath()");
  ret=getFwScriptPath();
  if(DEBUG) err_msg("DEBUG:(%s)<=getFwScriptPath()",ret);
  return ret;
}

char* GetFwScript(void)
{
  char* ret;
  if(DEBUG) err_msg("DEBUG:=>getFwScript()");
  ret=getFwScript();
  if(DEBUG) err_msg("DEBUG:(%s)<=getFwScript()",ret);
  return ret;
}

int GetUseFwScript(void)
{
  int ret;
  if(DEBUG) err_msg("DEBUG:=>getUseFwScript()");
  ret=getUseFwScript();
  if(DEBUG) err_msg("DEBUG:(%d)<=getUseFwScript()",ret);
  return ret;
}

